//bullet.cpp
#include "DXUT.h"
#include <btBulletDynamicsCommon.h>

btBroadphaseInterface*				g_broadphase = NULL;
btDefaultCollisionConfiguration*	g_collisionConfiguration = NULL;
btCollisionDispatcher*				g_dispatcher = NULL;
btConstraintSolver*					g_solver = NULL;
btDynamicsWorld*					g_world = NULL;		
btAlignedObjectArray<btCollisionShape*>	g_collisionShapes;

bool InitBullet()
{
	// broadphase 
	btVector3 worldAabbMin(-10000,-10000,-10000);
	btVector3 worldAabbMax(10000,10000,10000);
	g_broadphase = new btAxisSweep3( worldAabbMin, worldAabbMax, 100);
	
	// collision configuration 
	g_collisionConfiguration = new btDefaultCollisionConfiguration();

	// collision dispatcher 
	g_dispatcher = new btCollisionDispatcher(g_collisionConfiguration);

	// constraint solver 
	g_solver = new btSequentialImpulseConstraintSolver;

	// dynamics world 
	g_world = new btDiscreteDynamicsWorld( g_dispatcher, g_broadphase, g_solver, g_collisionConfiguration);
	g_world->setGravity( btVector3(0,-10,0) );
	return true;
}

void UnInitBullet()
{
	for (int i = g_world->getNumCollisionObjects()-1; i>=0 ;i--)
	{
		btCollisionObject* obj = g_world->getCollisionObjectArray()[i];
		btRigidBody* body = btRigidBody::upcast(obj);
		if (body && body->getMotionState())
		{
			delete body->getMotionState();
		}
		g_world->removeCollisionObject( obj );
		delete obj;
	}

	//
	for (int j=0;j<g_collisionShapes.size();j++)
	{
		btCollisionShape* shape = g_collisionShapes[j];
		delete shape;
	}

	//
	SAFE_DELETE( g_broadphase );
	SAFE_DELETE( g_dispatcher );
	SAFE_DELETE( g_solver );
	SAFE_DELETE( g_world );
	SAFE_DELETE( g_collisionConfiguration );
}

void CreateGround()
{
	// collision shape 
	btCollisionShape* groundShape = new btStaticPlaneShape( btVector3(0,1,0), 0);

	g_collisionShapes.push_back(groundShape);
	btTransform groundTransform;
	groundTransform.setIdentity();
	groundTransform.setOrigin(btVector3(0,0,0));

	btScalar mass(0);

	//rigidbody is dynamic if and only if mass is non zero, otherwise static
	bool isDynamic = (mass != 0.f);

	btVector3 localInertia(0,0,0);
	if (isDynamic)
		groundShape->calculateLocalInertia( mass, localInertia);

	// MotionState 
	btDefaultMotionState* myMotionState = new btDefaultMotionState(groundTransform);
	// construction info 
	btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,groundShape,localInertia);
	// rigid body 
	btRigidBody* body = new btRigidBody(rbInfo);

	// rigid body world ߰
	g_world->addRigidBody(body);
}

void CreateBox(const D3DXVECTOR3& vec)
{
	// collision shape 
	btVector3 v = btVector3( vec.x, vec.y, vec.z );
	btCollisionShape* colShape = new btBoxShape(btVector3(1,1,1));
	g_collisionShapes.push_back(colShape);
	
	btTransform startTransform;		
	startTransform.setIdentity();

	btScalar mass(10);
	
	bool isDynamic = (mass != 0.f);
	btVector3 localInertia(0,0,0);
	
	if (isDynamic)
		colShape->calculateLocalInertia(mass,localInertia);

	startTransform.setOrigin(v);

	// MotionState 
	btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
	// construction info 
	btRigidBody::btRigidBodyConstructionInfo rbInfo(mass,myMotionState,colShape,localInertia);
	// rigid body 
	btRigidBody* body = new btRigidBody(rbInfo);

	// rigid body world ߰
	g_world->addRigidBody(body);
}


void SimulationBullet(float fElapsedTime)
{
	if(g_world)
	{
		g_world->stepSimulation( fElapsedTime );
	}
}

int GetCollisionObjectSize()
{
	return g_world->getCollisionObjectArray().size();
}

D3DXMATRIX GetCollisionObjectTransform( int nIndex )
{
	btTransform trans = g_world->getCollisionObjectArray()[nIndex]->getWorldTransform();

	float mat[16];
	D3DXMATRIX matObj;
	trans.getOpenGLMatrix(mat);
	memcpy( &matObj, mat, sizeof(float)*16);
	return matObj;
}