//mesh.cpp
#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTmisc.h"
#include "DXUTCamera.h"
#include "DXUTSettingsDlg.h"
#include "SDKmisc.h"
#include "SDKmesh.h"
#include "bullet.h"


//main.cpp
extern ID3DXEffect*			g_pEffect;
extern CModelViewerCamera   g_Camera;


CDXUTXFileMesh*	g_pBox = NULL;
CDXUTXFileMesh*	g_pPlane = NULL;

HRESULT InitMesh( IDirect3DDevice9* pDevice, DWORD dwShaderFlags )
{
	HRESULT hr;
	WCHAR str[MAX_PATH];
    V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"Pick.fx" ) );
    V_RETURN( D3DXCreateEffectFromFile( pDevice, str, NULL, NULL, dwShaderFlags, 
                                        NULL, &g_pEffect, NULL ) );

    // Setup the camera's view parameters
    D3DXVECTOR3 vecEye(0.0f, 10.0f, -25.0f);
    D3DXVECTOR3 vecAt (0.0f, 0.0f, -0.0f);
    g_Camera.SetViewParams( &vecEye, &vecAt );


	g_pBox = new CDXUTXFileMesh;
	hr = g_pBox->Create( pDevice, L"box.x" );

	g_pPlane = new CDXUTXFileMesh;
	hr = g_pPlane->Create( pDevice, L"plane.x" );
	return S_OK;
}

void UnInitMesh()
{
	SAFE_DELETE( g_pBox );
	SAFE_DELETE( g_pPlane );
}

void RenderMesh( IDirect3DDevice9* pDevice )
{
    D3DXMATRIXA16 mWorld;
    D3DXMATRIXA16 mView;
    D3DXMATRIXA16 mProj;
    D3DXMATRIXA16 mWorldViewProjection;

    // Get the projection & view matrix from the camera class
    mWorld = *g_Camera.GetWorldMatrix();
    mProj = *g_Camera.GetProjMatrix();
    mView = *g_Camera.GetViewMatrix();
    mWorldViewProjection = mWorld * mView * mProj;

	D3DXMATRIX mObj = GetCollisionObjectTransform( 0 );
	
	mWorldViewProjection = mObj * mView * mProj;

	UINT iPass = 0;
	g_pEffect->Begin(&iPass, 0);

	g_pEffect->SetMatrix("g_mWorld",&mObj);
	g_pEffect->SetMatrix("g_mWorldViewProjection",&mWorldViewProjection);
	g_pEffect->SetVector( "g_MaterialDiffuseColor", &D3DXVECTOR4(0.5f,0.5f,0.2f,1.0f));

	g_pEffect->BeginPass( 0 );
	g_pPlane->Render( pDevice );
	g_pEffect->EndPass();

	for(int i=1;i<	GetCollisionObjectSize();i++)
	{
		D3DXMATRIX mObj = GetCollisionObjectTransform( i );
		
		mWorldViewProjection = mObj * mView * mProj;
		
		g_pEffect->SetMatrix("g_mWorld",&mObj);
		g_pEffect->SetMatrix("g_mWorldViewProjection",&mWorldViewProjection);
		g_pEffect->SetVector( "g_MaterialDiffuseColor", &D3DXVECTOR4(0, 0.8f, 0, 1));

		g_pEffect->BeginPass( 0 );
		g_pBox->Render( pDevice );
		g_pEffect->EndPass();
	}

	g_pEffect->End();
}